#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <limits.h>

#include "u.h"
#include "m.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitcmddisp.c,v 1.3 1997/12/15 18:53:43 kraemer Exp $"};

extern struct SFIT sFit;
/*
   --- Command processor for the FDISP command

   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitcmddisp.c,v $
   Revision 1.3  1997/12/15 18:53:43  kraemer
   memcpy() bug fixed

   Revision 1.2  1997/08/29 12:08:55  kraemer
   gfxattr rework

   Revision 1.1.1.1  1997/07/10 17:59:58  kraemer
   revision for cvs

*/

void fitCmdDisp(
                const char *pcFct,
                long *plSingly, long *plPeaks,
                const char *pcColours,
                const char *pcWidths,
                const char *pcLineTypes,
                long *plPts, const char *pcWdw,
                double *pdConf,
                long *plDebug
               ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__, *pMsg = pDbgMsg;
   unsigned long lFlag = FIT_NOMSG, lFlagG = 0;

   struct SFITWDW sFW;
   struct SFITDATA sFD;
   struct SFITGFX sFGfx;
   struct SGFXATTR sGA;
   struct SFITDISPATTR sFDA;
   struct SFITFCTBLK sFFB, sFFBPeaks, sFFBSingle, *psFFB;

   char **pcCl = NULL, **pcLM = NULL, **pcLW = NULL;
   int   *ncCl =    0,  *ncLM =    0,  *ncLW =    0;
   int nCl, nLM, nLW;

   long *pvlFct = NULL; int iFct, nFct; long lFct;

   long ll; double dLim[2];

   int iWdw=0, nWdw=0;
   char **pvcWdw = NULL;
   int  *nvcWdw = NULL;

   char *pc;
   int ii, jj, nn;



   if ( *plDebug ) lFlag  |= FIT_DEBUG;
   if ( *plDebug ) lFlagG |= G_DEBUG;
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
   memset( &sFW, 0, sizeof(sFW) );
   memset( &sFD, 0, sizeof(sFD) );
   memset( &sFDA, 0, sizeof(sFDA) );
   memset( &sFFB, 0, sizeof(sFFB) );
   memset( &sFFBPeaks, 0, sizeof(sFFBPeaks) );
   memset( &sFFBSingle, 0, sizeof(sFFBSingle) );
   memset( &sFGfx, 0, sizeof(sFGfx) );
                                      /* check valid inputs */
   if ( *pdConf > 0E0 ) {
      if ( *pdConf >= 1E0 ) {
         iRC = ERRFIT_INVCONF; goto gError;
      }
   }
                                      /* check prerequisites */
   if ( sFit.sFFB.nf <= 0 ) {
      iRC = ERRFIT_NOFUNCTION; goto gError;
   }
   if ( sFit.eFitGfxAttr == NULL ) {
      iRC = ERRFIT_NOGFXAVAIL; goto gError;
   }
   if ( sFit.eFitWdw == NULL ) {
      iRC = ERRFIT_NOWDWAVAIL; goto gError;
   }
   if ( (sFGfx.pGH = _GFndWdw( sFit.pcWdw )) == NULL ) {
      iRC = ERRFIT_NOGFXAVAIL; goto gError;
   }
/*
   --- for confidence limits, calculate complete covariance matrix here
*/
   if ( *pdConf > 0E0 ) {
      unsigned long lFlagCurve = 0;

      if ( sFit.sFDFit.nn <= 0 ) {
         iRC = ERRFIT_NODATA; goto gError;
      }
      if ( sFit.lStatus & FIT_POISSON ) lFlagCurve |= FITFCTBLK_POISSON;
                                      /* function values + derivatives */
      if ( iRC = fitDataFctBlkDeriv( &sFit.sFDFit, &sFit.sFFB, lFlag ) ) goto gError;
                                      /* curvature matrices */
      if ( iRC = fitDataFctBlkCurv ( &sFit.sFDFit, &sFit.sFFB, lFlag | lFlagCurve ) ) goto gError;
                                      /* covariance matrix */
      if ( iRC = fitFctBlkCovar(                   &sFit.sFFB, lFlag ) ) goto gError;
   }
/*
   --- Handle functions
*/
   while ( *pcFct == ' ' ) pcFct++;
                                      /* all functions ? */
   if ( (*pcFct == '*') || (*pcFct == '\0') ) {
/*
      if ( iRC = fitFctBlkDup( &sFit.sFFB, &sFFB, lFlag ) ) goto gError;
*/
      psFFB = &sFit.sFFB; nFct = psFFB->nf;
   } else {                           /* list of functions */
      if ( (nFct = c2vl( pcFct, pvlFct, 1, INT_MAX )) <= 0 ) {
         pMsg = (char *) pcFct; iRC = ERRFIT_CONVNAN; goto gError;
      }
      if ( (pvlFct = calloc( nFct, sizeof(long) )) == NULL ) {
         pMsg = (char *) pcFct; iRC = ERRFIT_SYS; goto gError;
      }
      if ( (nFct != c2vl( pcFct, pvlFct, 1, nFct )) ) {
         pMsg = (char *) pcFct; iRC = ERRFIT_CONVNAN; goto gError;
      }
      if ( (iRC = fitFctBlkByNum( &sFit.sFFB, pvlFct, nFct, &sFFB, lFlag )) != 0 ) {
         pMsg = (char *) pcFct; goto gError;
      }
      psFFB = &sFFB;
   }
                                      /* build single peak function */
   if ( *plPeaks ) {
      if ( iRC = fitFctBlkPeaks( psFFB, &sFFBPeaks, lFlag ) )
         goto gError;
      psFFB = &sFFBPeaks; nFct = psFFB->nf;
   }
/*
   --- Display attributes
*/
   iRC = ERRFIT_SYS;
   if ( ( pcCl = calloc( nFct, sizeof(char *) ) ) == NULL ) goto gError;
   if ( ( ncCl = calloc( nFct, sizeof( int  ) ) ) == NULL ) goto gError;
   if ( ( pcLM = calloc( nFct, sizeof(char *) ) ) == NULL ) goto gError;
   if ( ( ncLM = calloc( nFct, sizeof( int  ) ) ) == NULL ) goto gError;
   if ( ( pcLW = calloc( nFct, sizeof(char *) ) ) == NULL ) goto gError;
   if ( ( ncLW = calloc( nFct, sizeof( int  ) ) ) == NULL ) goto gError;
                                      /* decompose */
   iRC = ERRFIT_BADATTRLISTSPEC;
   while( *pcColours == ' ' ) pcColours++;
   if ( (nCl=c2vcn( (char *) pcColours, pcCl, ncCl, 1, nFct )) < 0 ) {
      pMsg = (char *) pcColours; goto gError;
   }
   if ( nCl == 0 ) { pcCl[nCl++] = ""; }
   while( *pcWidths == ' ' ) pcWidths++;
   if ( (nLW=c2vcn( (char *) pcWidths, pcLW, ncLW, 1, nFct )) < 0 ) {
      pMsg = (char *) pcWidths; goto gError;
   }
   if ( nLW == 0 ) { pcLW[nLW++] = ""; }
   while( *pcLineTypes == ' ' ) pcLineTypes++;
   if ( (nLM=c2vcn( (char *) pcLineTypes, pcLM, ncLM, 1, nFct )) < 0 ) {
      pMsg = (char *) pcLineTypes; goto gError;
   }
   if ( nLM == 0 ) { pcLM[nLM++] = ""; }
                                      /* cyclic padding */
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> ncol=%d\n", nCl );
   jj = 0;
   for ( ii=nCl; ii<nFct; ii++ ) {
      pcCl[ii] = pcCl[jj];
      ncCl[ii] = ncCl[jj++];
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> nLW=%d\n", nLW );
   jj = 0;
   for ( ii=nLW; ii<nFct; ii++ ) {
      pcLW[ii] = pcLW[jj];
      ncLW[ii] = ncLW[jj++];
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> nLM=%d\n", nLM );
   jj = 0;
   for ( ii=nLM; ii<nFct; ii++ ) {
      pcLM[ii] = pcLM[jj];
      ncLM[ii] = ncLM[jj++];
   }
if ( lFlag & FIT_DEBUG ) {
   for ( ii=0; ii<nFct; ii++ ) {
      fprintf( stderr, "<D> colour=%*.*s lw=%*.*s lm=%*.*s\n",
                       ncCl[ii], ncCl[ii], pcCl[ii],
                       ncLW[ii], ncLW[ii], pcLW[ii],
                       ncLM[ii], ncLM[ii], pcLM[ii]
             );
   }
}
/*
   --- handle windows
*/
                                      /* default window limits */
   pc = "#";
   if ( sFit.sFD.lFlag & FITDATA_DATALIM ) pc = "*";
   if ( sFit.eFitWdw( pc, strlen(pc), &sFW, 0 ) < 0 ) {
      pMsg = pc;
      iRC = ERRFIT_NOSUCHWDW; goto gError;
   }
                                      /*  */
   while ( *pcWdw == ' ' ) pcWdw++;
   if ( *pcWdw ) {
                                      /* decompose window parameter */
      if ( ( nWdw = c2vc( (char *) pcWdw, NULL, 1, INT_MAX ) ) <= 0 ) {
         pMsg = (char *) pcWdw;
         iRC = ERRFIT_BADWDWLISTSPEC; goto gError;
      }
      if ( ( pvcWdw = calloc( nWdw, sizeof(char **) ) ) == NULL ) {
         pMsg = (char *) pcWdw;
         iRC = ERRFIT_SYS; goto gError;
      }
      if ( ( nvcWdw = calloc( nWdw, sizeof(int *) ) ) == NULL ) {
         pMsg = (char *) pcWdw;
         iRC = ERRFIT_SYS; goto gError;
      }
      if ( c2vcn( (char *) pcWdw, pvcWdw, nvcWdw, 1, nWdw ) != nWdw ) {
         pMsg = (char *) pcWdw;
         iRC = ERRFIT_BADWDWLISTSPEC; goto gError;
      }
      if ( nWdw == 1 ) {              /* single item: wdw,cond,... */
         memset( &sFW, 0, sizeof(sFW) );
         if ( sFit.eFitWdw( pvcWdw[iWdw], nvcWdw[iWdw], &sFW, lFlag ) < 0 ){
            pMsg = pvcWdw[iWdw];
            iRC = ERRFIT_NOSUCHWDW; goto gError;
         }
      } else {                        /* pair of items: assume x-number list */
         double *pd = (double *) sFW.dLim; int mDim = MFITWDWDIM*2;

         while( nWdw-- && mDim--) {
                                      /* '*': keep default limits */
            if ( strncmp( pvcWdw[iWdw], "*", nvcWdw[iWdw] ) != 0 ) {
               if ( cn2d( pvcWdw[iWdw], nvcWdw[iWdw], pd ) < 0 ) {
                  pMsg = (char *) pcWdw;
                  iRC = ERRFIT_CONVNAN; goto gError;
               }
            }
            iWdw++; pd++;
         }
      }
   }
   sFW.nDim = 1;
/*
   ---
*/
                                      /* determine # of points */
   iRC = ERRFIT_SYS;
   if ( (nn=*plPts) <= 0 ) {
      if ( ( nn = _GBrkPnt(sFGfx.pGH,lFlagG) ) <= 0 ) return;
   }
   if ( iRC = fitDataAlloc( nn, &sFD, lFlag ) ) goto gError;
                                      /* compute/scale x-values */
   dLim[0] = sFW.dLim[0][0];
   dLim[1] = sFW.dLim[0][1];
   if ( sFit.sFD.lFlag & FITDATA_DATALIM ) {
      dLim[0] =  DBL_MAX;
      dLim[1] = -DBL_MAX;
      minv( sFit.sFD.pdD[FIT_DIMX][FITDATA_DATA], 1, dLim+0, &ll, sFit.sFD.nn );
      maxv( sFit.sFD.pdD[FIT_DIMX][FITDATA_DATA], 1, dLim+1, &ll, sFit.sFD.nn );
   }
   _GRamp( sFGfx.pGH, dLim, lFlagG | G_COORDLOG,
           sFD.pdD[FIT_DIMX][FITDATA_DATA], 1, nn );
   sFD.sFCalX = sFit.sFCalX;
   sFD.lFlag = sFit.sFD.lFlag;
   memcpy( sFD.lScale, sFit.sFD.lScale, sizeof(sFD.lScale) );
   sFD.dAvgBinsize = sFit.sFD.dAvgBinsize;
   if ( iRC = fitDataScale( &sFD, FIT_DIMX, lFlag ) ) goto gError;
                                      /* clip plot to axes */
   _GClpRct( sFGfx.pGH, NULL, lFlagG | GCLP_INT );
/*
   --- compute/display functions
*/
                                      /*  */
   if ( *plSingly || *plPeaks ) {     /* draw functions/peaks singly */
                                      /* function loop */
      for ( iFct=0; iFct<nFct; iFct++ ) {
                                      /* get/set attributes */
         sFDA.pcColour = pcCl[ iFct ];
         sFDA.ncColour = ncCl[ iFct ];
         sFDA.pcLMode  = pcLM[ iFct ];
         sFDA.ncLMode  = ncLM[ iFct ];
         sFDA.pcLWidth = pcLW[ iFct ];
         sFDA.ncLWidth = ncLW[ iFct ];
         sFDA.pcHMode  = "";
         sFDA.pcHWidth = "";
         sGA = sFit.sGA;
         if ( sFit.eFitGfxAttr ) sFit.eFitGfxAttr( &sFDA, &sGA, lFlag );
         _GGfxAttrSet( sFGfx.pGH, &sGA,
                       GGFXATTR_COLOUR | GGFXATTR_FCT |
                       GGFXATTR_LMODE | GGFXATTR_LWIDTH | lFlagG );
                                      /* a single function block */
         lFct = iFct+1;
         if ( iRC = fitFctBlkByNum( psFFB, &lFct, 1, &sFFBSingle, lFlag ) )
            goto gError;
         if ( iRC = fitDataFctBlkValue( &sFD, &sFFBSingle, lFlag ) ) goto gError;
                                      /* scale for display */
         if ( iRC = fitDataScaleFct( &sFD, FITDATA_SCALEFCT, lFlag ) ) goto gError;
                                      /* finall, display it */
         sFGfx.lFlag = lFlagG | G_COORDLOG;
         if ( iRC = fitDataGfxLine ( &sFD, &sFGfx, lFlag ) ) goto gError;
                                      /* confidence limits */
         if ( *pdConf > 0E0 ) {
                                      /* compute confidence  */
            if ( iRC=fitDataFctBlkConf( &sFD, &sFFBSingle, *pdConf, lFlag ) )
               goto gError;
            if ( iRC = fitDataGfxLine ( &sFD, &sFGfx, lFlag | FITDATAGFXLINE_CONF ) ) goto gError;
         }
         fitFctBlkFree( &sFFBSingle, lFlag );
      } /* iFct */
   } else {                           /* all functions */
                                      /* get/set attributes */
      iFct = 0;
      sFDA.pcColour = pcCl[ iFct ];
      sFDA.ncColour = ncCl[ iFct ];
      sFDA.pcLMode  = pcLM[ iFct ];
      sFDA.ncLMode  = ncLM[ iFct ];
      sFDA.pcLWidth = pcLW[ iFct ];
      sFDA.ncLWidth = ncLW[ iFct ];
      sFDA.pcHMode  = "";
      sFDA.pcHWidth = "";
      sGA = sFit.sGA;
/*
fprintf( stderr, "<D> col=%*.*s\n",
                 sFDA.ncColour, sFDA.ncColour, sFDA.pcColour );
*/
      if ( sFit.eFitGfxAttr ) sFit.eFitGfxAttr( &sFDA, &sGA, lFlag );
/*
fprintf( stderr, "<D> col=%d %d %d %d\n",
                 sGA.lRGB[0], sGA.lRGB[1], sGA.lRGB[2], sGA.lColour );
*/
      _GGfxAttrSet( sFGfx.pGH, &sGA,
                    GGFXATTR_COLOUR | GGFXATTR_FCT |
                    GGFXATTR_LMODE | GGFXATTR_LWIDTH | lFlagG );
      sFGfx.lFlag = lFlagG | G_COORDLOG;
                                      /* function values */
      if ( iRC = fitDataFctBlkValue( &sFD, psFFB, lFlag ) ) goto gError;
                                      /* scale for display */
      if ( iRC = fitDataScaleFct( &sFD, FITDATA_SCALEFCT, lFlag ) ) goto gError;
                                      /* finally, display it */
      if ( iRC = fitDataGfxLine ( &sFD, &sFGfx, lFlag ) ) goto gError;
                                      /* confidence limits */
      if ( *pdConf > 0E0 ) {
                                      /* compute confidence  */
         if ( iRC=fitDataFctBlkConf( &sFD, &sFit.sFFB, *pdConf, lFlag ) )
            goto gError;
         if ( iRC = fitDataGfxLine ( &sFD, &sFGfx, lFlag | FITDATAGFXLINE_CONF ) ) goto gError;
      }
   }

   iRC = 0;
gError:;
   if ( (iRC < 0 ) /*&& !(lFlag&FIT_NOMSG)*/ ) {
      fitemsg( pMsg, iRC );
   }
                                      /* cleanup */
   fitFctBlkFree( &sFFBSingle, lFlag );
   fitDataFree( &sFD, lFlag );
   if ( pvcWdw ) free( pvcWdw ); pvcWdw = NULL;
   if ( nvcWdw ) free( nvcWdw ); nvcWdw = NULL;
   if ( pcCl ) free(pcCl); pcCl = NULL;
   if ( pcLW ) free(pcLW); pcLW = NULL;
   if ( pcLM ) free(pcLM); pcLM = NULL;
   if ( ncCl ) free(ncCl); ncCl = NULL;
   if ( ncLW ) free(ncLW); ncLW = NULL;
   if ( ncLM ) free(ncLM); ncLM = NULL;
   fitFctBlkFree( &sFFB, lFlag );
   fitFctBlkFree( &sFFBPeaks, lFlag );
   if ( pvlFct ) free(pvlFct); pvlFct = NULL;

if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return;
}

