
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <limits.h>

#include "u.h"
#include "m.h"
#include "polfit.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitcmdexp.c,v 1.2 1997/08/29 12:09:22 kraemer Exp $"};

extern struct SFIT sFit;
/*
   --- Command processor for the FEXP command

   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitcmdexp.c,v $
   Revision 1.2  1997/08/29 12:09:22  kraemer
   gfxattr rework

   Revision 1.1.1.1  1997/07/10 17:59:58  kraemer
   revision for cvs

*/

void fitCmdExp(
                const char *pcNum,
                long *plPrompt, long *plNoDisp,
                double *pdBackGround,
                long *plDebug
              ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__, *pMsg = pDbgMsg;
   unsigned long lFlag = 0, lFlagG = 0;

   unsigned long lType = 0;
   long lExp, mExp = 1;
   double dd;
   int ip, np = 0;

   struct SFITPARDERIV sFPD;
   struct SFITDATA sFD, sFDPos;
   struct SFITGFX sFGfx;
   struct SFITFCT sFF, sFFBG;
   struct SFITFCTBLK sFFB;
   struct SFITWDW sFW, sFWDef, sFWPos, *psFW;
   struct SFITWDWNODE *psFWN = NULL;

   struct List sLFW;
   struct Node *psln;

   char cStr[80+1];


   if ( *plDebug ) lFlag |= FIT_DEBUG;
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
   if ( lFlag & FIT_DEBUG ) lFlagG |= G_DEBUG;
   memset( &sFGfx, 0, sizeof(sFGfx) );
   memset( &sFD, 0, sizeof(sFD) );
   memset( &sFDPos, 0, sizeof(sFDPos) );
   memset( &sFF, 0, sizeof(sFF) );
   memset( &sFFB, 0, sizeof(sFFB) );
   memset( &sFFBG, 0, sizeof(sFFBG) );
   memset( &sFWDef, 0, sizeof(sFWDef) );
   memset( &sFPD, 0, sizeof(sFPD) );
   NewList( &sLFW );

                                      /* check window interface exists */
   if ( sFit.eFitWdw == NULL ) {
      iRC = ERRFIT_NOWDWAVAIL; goto gError;
   }
                                      /* get display limits */
   if ( sFit.eFitWdw( "#", 1, &sFWDef, lFlag ) < 0 ) {
      pMsg = "#";
      iRC = ERRFIT_NOSUCHWDW; goto gError;
   }
                                      /* current display handle */
   sFGfx.pGH = _GFndWdw( sFit.pcWdw );
   if ( lFlag & FIT_DEBUG ) sFGfx.lFlag |= G_DEBUG;

                                      /* # of exponentials */
   while( *pcNum == ' ' ) pcNum++;
   if ( *pcNum == '*' ) {             /* determined by fit windows */
      mExp = 0;
                                      /* duplicate */
      if ( iRC = fitWdwListDup( &sFit.sListFW, &sLFW, lFlag ) ) goto gError;
      for( psln = sLFW.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ) {
         mExp++;
      }
      *plPrompt = 0;                  /* no prompting */
      if ( mExp <= 0 ) {              /* if no windows: some default */
                                      /* duplicate window node */
         if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
            iRC = ERRFIT_SYS; goto gError;
         }
         psFWN->sFW = sFWDef;
                                      /* link into new list */
         AddTail( &sLFW, &psFWN->ln ); psFWN = NULL;
         mExp = 1;
      }
   } else {                           /* determined directly as number */
      if ( c2l( pcNum, &mExp ) != 0 ) {
         pMsg = (char *) pcNum;
         iRC = ERRFIT_CMDEXPORDER; goto gError;
      }
   }

   np = mExp * 2 + 1; lType = FITFCT_EXP;
                                      /* */
                                      /* allocate local function  */
   if ( ( iRC = fitFctAlloc( np, &sFF, lFlag) ) != 0 ) goto gError;
                                      /* fill entries */
   fitFctInit( &sFF, lFlag);
   sprintf( cStr, "%d exponentials + constant term", mExp );
   sFF.pcName = strdup("Exponential");
   sFF.eCall  = fitExpon;

   if ( ( sFF.pcText = strdup(cStr) ) == NULL ) {
      iRC = ERRFIT_SYS; goto gError;
   }
                                      /* early initialize parameters */
   fitDataFctInitPar( &sFit.sFD, &sFF, lFlag );
                                      /* prompt for windows */
   if ( *plPrompt ) {
      for ( lExp=0; lExp<mExp; lExp++ ) {
         iRC = fitWdwPrompt( &sFW, &sFGfx, lFlag );
         if ( iRC > 0 ) break;
         if ( iRC < 0 ) goto gError;
                                      /* duplicate window node */
         if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
            iRC = ERRFIT_SYS; goto gError;
         }
         psFWN->sFW = sFW;
                                      /* link into new list */
         AddTail( &sLFW, &psFWN->ln ); psFWN = NULL;
      }
   }
                                      /* if no window list,
                                         use some arbitrary partition */
   if ( sLFW.lh_Head -> ln_Succ == NULL ) {
      sFW = sFWDef;
      dd = ( sFW.dLim[0][1] - sFW.dLim[0][0] ) / (double) mExp;
      for ( lExp=0; lExp<mExp; lExp++ ) {
         sFW.dLim[0][1] = sFW.dLim[0][0] + dd;
                                      /* duplicate window node */
         if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
            iRC = ERRFIT_SYS; goto gError;
         }
         psFWN->sFW = sFW;
                                      /* link into new list */
         AddTail( &sLFW, &psFWN->ln ); psFWN = NULL;
         sFW.dLim[0][0] = sFW.dLim[0][1];
      }
   }
                                      /* estimate parameters from data */
   if ( sFit.sFD.nn > 0 ) {
                                      /* apply global error modes */
      if ( sFit.sFDE.lMode > 0 ) {
         sFit.sFD.sFDE[FIT_DIMY] = sFit.sFDE;
      } else {
/*
         sFit.sFDE = sFit.sFD.sFDE[FIT_DIMY];
*/
      }
      if ( ( iRC = fitDataErr( &sFit.sFD, FIT_DIMX, lFlag ) ) != 0 )
         goto gError;
      if ( ( iRC = fitDataErr( &sFit.sFD, FIT_DIMY, lFlag ) ) != 0 )
         goto gError;
                                      /* scale the data */
      sFit.sFD.sFCalX = sFit.sFCalX;
      if ( ( iRC = fitDataScale( &sFit.sFD, FIT_DIMX, lFlag ) ) != 0 ) goto gError;
      if ( ( iRC = fitDataScale( &sFit.sFD, FIT_DIMY, lFlag ) ) != 0 ) goto gError;
                                      /* take into account background */
                                      /* create local function  */
      if ( ( iRC = fitFctAlloc( np=1, &sFFBG, lFlag) ) != 0 ) goto gError;
      fitFctInit( &sFFBG, lFlag);
      sprintf( cStr, "constant background" );
      sFFBG.pcName = strdup("Polynomial");
      sFFBG.eCall  = fitPoly;
      if ( ( sFFBG.pcText = strdup(cStr) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }
      sFFBG.pdVal[0] = *pdBackGround;
      if ( iRC = fitFctBlkFctPut( &sFFB, &sFFBG, lFlag) ) goto gError;
                                      /* compute value & difference */
      if ( iRC = fitDataFctBlkValue( &sFit.sFD, &sFFB, lFlag ) ) goto gError;
      if ( iRC = fitDataScaleFct   ( &sFit.sFD, FITDATA_SCALEFCT, lFlag) ) goto gError;
      fitDataNetScaled( &sFit.sFD, lFlag );
      if ( iRC = fitDataScaleFct   ( &sFit.sFD, FITDATA_SCALEDIFF, lFlag) ) goto gError;
                                      /* prepare wdw to exclude nonpositive */
      fitWdwInit( &sFWPos, lFlag );
      sFWPos.nDim = 2; sFWPos.dLim[FIT_DIMY][0] = DBL_MIN;
                                      /* loop over defining windows */
      ip = 1;
      for( psln = sLFW.lh_TailPred; psln->ln_Pred; psln=psln->ln_Pred ) {
                                      /* window excluding <= 0 */
         ( (struct SFITWDWNODE *) psln ) -> sFW.nDim = sFWPos.nDim;
         if ( ( iRC = fitWdwAnd( &sFWPos,
                                 &( ( (struct SFITWDWNODE *) psln ) -> sFW ),
                                 &sFW, lFlag ) ) != 0 )
            goto gError;
                                      /* collect data in window */
         if ( ( iRC = fitDataWdw( &sFit.sFD,
                                  &sFW,
                                  &sFD, lFlag) ) != 0 ) goto gError;
                                      /* collect positive data in window */
         vmov( sFD.pdR[FIT_DIMY][FITDATA_DATA], 1,
               sFD.pdD[FIT_DIMY][FITDATA_DATA], 1,
               sFD.nn );
         vmov( sFD.pdR[FIT_DIMY][FITDATA_ERRM], 1,
               sFD.pdD[FIT_DIMY][FITDATA_ERRM], 1,
               sFD.nn );
         vmov( sFD.pdR[FIT_DIMY][FITDATA_ERRP], 1,
               sFD.pdD[FIT_DIMY][FITDATA_ERRP], 1,
               sFD.nn );
         if ( ( iRC = fitDataWdw( &sFD,
                                  &sFW,
                                  &sFDPos, lFlag) ) != 0 ) goto gError;
                                      /* linear fit estimate */
         if ( sFD.nn > 2 ) {
            struct SPOLFITPAR sPFP;
            struct SPOLFITDATA sPFD;
            unsigned long lFlagPF = 0;


            if ( lFlag & FIT_DEBUG ) lFlagPF |= POLFIT_DEBUG;

            if ( polfitParAlloc( 2, &sPFP, lFlagPF ) != 0 ) {
               iRC = ERRFIT_LSQPOL; goto gError;
            }
            if ( polfitDataAlloc( sFDPos.nn, &sPFD, lFlagPF ) != 0 ) {
               polfitParFree( &sPFP, lFlagPF );
               iRC = ERRFIT_LSQPOL; goto gError;
            }

            vmov( sFDPos.pdD[FIT_DIMX][FITDATA_DATA], 1,
                  sPFD.pdX, 1,
                  sFDPos.nn );
            vlog( sFDPos.pdR[FIT_DIMY][FITDATA_DATA], 1,
                  sPFD.pdY, 1,
                  sFDPos.nn );
            vmov( sFDPos.pd [FIT_DIMY][FITDATA_ERRM], 1,
                  sPFD.pddYm, 1,
                  sFDPos.nn );
            vmov( sFDPos.pd [FIT_DIMY][FITDATA_ERRP], 1,
                  sPFD.pddYp, 1,
                  sFDPos.nn );
/*
            if ( vlsqpol( sFDPos.pdD[FIT_DIMX][FITDATA_DATA],
                          sFDPos.pdR[FIT_DIMY][FITDATA_DATA],
                          pdW, sFDPos.pdAux[1], sFDPos.nn,
                          dvPar, sFPD.pd, dvCovar, 2, &dd
                        ) != 0 ) {
               iRC = ERRFIT_LSQPOL; goto gError;
            }
*/
            if ( polfitFit( &sPFD, &sPFP, lFlagPF ) != 0 ) {
               polfitParFree ( &sPFP, lFlagPF );
               polfitDataFree( &sPFD, lFlagPF );
               iRC = ERRFIT_LSQPOL; goto gError;
            }

            sFF.pdVal[ip+0] = exp( sPFP.pdPar[0] );
            sFF.pdVal[ip+1] =      sPFP.pdPar[1];

            polfitParFree ( &sPFP, lFlagPF );
            polfitDataFree( &sPFD, lFlagPF );
         }

         fitDataFree( &sFDPos, lFlag );
         fitDataFree( &sFD, lFlag );
         ip += 2;
      }
      fitFctBlkFree( &sFFB, lFlag);
   }
                                      /* constant background */
   sFF.pdVal[0] = *pdBackGround;
                                      /* add to function block */
   if ( ( iRC = fitFctBlkFctPut( &sFit.sFFB, &sFF, lFlag) ) != 0 ) goto gError;

                                      /* display */
   if ( !*plNoDisp && ( sFit.sFD.nn > 0 ) && sFGfx.pGH ) {
                                      /* build local function block */
      if ( ( iRC = fitFctBlkFctPut( &sFFB, &sFF, lFlag) ) != 0 ) goto gError;
      if ( ( iRC = fitDataWdw( &sFit.sFD, &sFWDef, &sFD, lFlag) ) != 0 ) goto gError;
                                      /* compute function */
      if ( ( iRC = fitDataFctBlkValue( &sFD, &sFFB, lFlag) ) != 0 ) goto gError;
      if ( ( iRC = fitDataScaleFct( &sFD, FITDATA_SCALEFCT, lFlag) ) != 0 ) goto gError;
                                      /* finally, display it */
      sFGfx.lFlag = lFlagG | G_COORDLOG;
      _GGfxAttrSet( sFGfx.pGH, &sFit.sGA,
                    GGFXATTR_COLOUR | GGFXATTR_FCT |
                    GGFXATTR_LMODE | GGFXATTR_LWIDTH | lFlagG );
      if ( iRC = fitDataGfxLine ( &sFD, &sFGfx, lFlag ) ) goto gError;
   }

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
                                      /* cleanup */
   fitDataFree( &sFDPos, lFlag );
   fitDataFree( &sFD, lFlag );
   fitFctBlkFree( &sFFB, lFlag );
   fitWdwListFree( &sLFW, lFlag );
   if ( psFWN ) free( psFWN ); psFWN = NULL;
   fitFctFree( &sFF, lFlag);
   fitFctFree( &sFFBG, lFlag);

if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return;
}
