
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <limits.h>

#include "u.h"
#include "m.h"
#include "btree.h"
#include "cmd.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitcmdsum.c,v 1.2 2000/04/03 12:57:51 kraemer Exp $"};

extern struct SFIT sFit;
/*
   --- Command processor for the FSUM and FLIST command

   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitcmdsum.c,v $
   Revision 1.2  2000/04/03 12:57:51  kraemer
   *** empty log message ***

   Revision 1.1.1.1  1997/07/10 18:00:01  kraemer
   revision for cvs

*/

enum { FITCMDSUM_XMIN=0, FITCMDSUM_XMAX, FITCMDSUM_SUM, FITCMDSUM_ERRSUM,
       FITCMDSUM_MEAN, FITCMDSUM_VAR, FITCMDSUM_SKEW, MSUMITEM
};

enum { FITCMDSUM_TOTAL=0, FITCMDSUM_NET=1, MPASS };

void fitCmdSum( const char *pcWdw,
                long *plDisp, long *plCond,
                long *plLoop, long *plAll, long *plGraphic,
                long *plNet, long *plTotal, long *plFWHM, long *plFitErr,
                double *pdConf,
                const char *pcStore,
                long *plSwitch, 
                long *plDebug
               ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__, *pMsg = pDbgMsg;
   unsigned long lFlag = 0, lFlagI = 0, lFlagCurve = 0;

   struct SFITMOM sFMomPos, sFMomNeg;
   struct SFITDATA sFD;
   struct SFITWDWNODE *psFWN = NULL;
   struct SFITWDW sFWDef, sFW, *psFW;
   struct List sLFWN;
   struct Node *psln;

   struct List sLTxt;
   char cStr[255+1] = { "" } ;
   char cDispWdw[2] = { 0,0 };
   char cCnd[20] = { "" };

   static char *cSum[] = { "Total Sum", "Net Sum" };
   static char *cVar[] = { "Variance", "FWHM" };
   static char *cSkw[] = { "Skewness", "Rel.Skew" };

   int iPass, nPass, iMod;
   unsigned long lFlagPrtcl = CMDPRTCL_FILE | CMDPRTCL_TERM;
   unsigned long lMomAct = 0;

   int ii;
   int iWdw, nWdw;
   int   *nvcWdw = NULL;
   char **pvcWdw = NULL;

   int nItem;
   int nvItem[3] = {0,0,0};
   double ***dvItem = NULL;


   if ( *plDebug ) lFlag |= FIT_DEBUG;
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
   if ( lFlag & FIT_DEBUG ) lFlagI |= CMD_DEBUG;
   if ( sFit.lStatus & FIT_POISSON ) lFlagCurve |= FITFCTBLK_POISSON;
   memset( &sFD, 0, sizeof(sFD) );
   memset( &sFWDef, 0, sizeof(sFWDef) );
   NewList( &sLFWN );
   NewList( &sLTxt );
                                      /* check prerequisites */
   if ( sFit.sFDFit.nn <= 0 ) {
      iRC = ERRFIT_NODATA; goto gError;
   }
                                      /* check valid inputs */
   if ( *plAll) {
      pMsg = "all";
      iRC = ERRFIT_CMDPARNOTYET; goto gError;
   }
/*
   while ( *pcStore == ' ' ) pcStore++;
   if ( *pcStore ) {
      pMsg = "store()";
      iRC = ERRFIT_CMDPARNOTYET; goto gError;
   }
*/
   if ( *pdConf >= 1E0 ) {
      iRC = ERRFIT_INVCONF; goto gError;
   }
   if ( *pdConf <= 0E0 ) {
      iRC = ERRFIT_INVCONF; goto gError;
   }
                                      /* */
   if ( ( *plLoop + *plDisp + *plCond + *plAll ) > 1 ) {
      pMsg = "loop/disp/cond/all";
      iRC = ERRFIT_CMDPAREXCL; goto gError;
   }
   if ( *plGraphic ) {
                                      /* check graphic interface exists */
      if ( sFit.eFitGfxTxt == NULL ) {
         iRC = ERRFIT_GFXTXTNOTAVAIL; goto gError;
      }
   }

                                      /* FWHM modifier */
   iMod = 0;
   if ( *plFWHM ) iMod++;
/*
   --- Setup list of windows
*/
                                      /* check window interface exists */
   if ( sFit.eFitWdw == NULL ) {
      iRC = ERRFIT_NOWDWAVAIL; goto gError;
   }
                                      /* fetch default limits */
   memset( &sFWDef, 0, sizeof(sFWDef) );
   if ( sFit.eFitWdw( "#", 1, &sFWDef, lFlag ) < 0 ) {
      pMsg = "#";
      iRC = ERRFIT_NOSUCHWDW; goto gError;
   }
                                      /* skip blanks */
   while( *pcWdw == ' ' ) pcWdw++;
   switch( *pcWdw ) {
                                      /* fit windows */
   case '*':
      if ( sFit.sListFW.lh_Head -> ln_Succ ) {
                                      /* duplicate fit window list */
         if ( ( iRC = fitWdwListDup( &sFit.sListFW, &sLFWN, lFlag ) ) != 0 )
            goto gError;
         else;
      } else {                        /* at least display region */
                                      /* duplicate window */
         if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
            iRC = ERRFIT_SYS; goto gError;
         }
         psFWN->sFW = sFWDef;
                                      /* link into new list */
         AddTail( &sLFWN, &psFWN->ln ); psFWN = NULL;
      }
   break;

   case '\0':
                                      /* nothing specified: loop is default */
      if ( (*plAll + *plDisp + *plCond + *plLoop) == 0 ) *plLoop =1;
                                      /* prompt window by cursor/terminal */
      if ( *plLoop ) {
         struct SFITGFX sFGfx;

         memset( &sFGfx, 0, sizeof(sFGfx) );
         if( lFlag & FIT_DEBUG ) sFGfx.lFlag = G_DEBUG;
         sFGfx.pGH = _GFndWdw( sFit.pcWdw );

         sFW = sFWDef;                /* default = display limit */
         while( (iRC = fitWdwPrompt( &sFW, &sFGfx, lFlag )) == 0 ) {
                                      /* duplicate window node */
            if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
               iRC = ERRFIT_SYS; goto gError;
            }
            psFWN->sFW = sFW;
                                      /* link into new list */
            AddTail( &sLFWN, &psFWN->ln ); psFWN = NULL;
            sFW = sFWDef;
         }
                                      /* error exit */
         if ( iRC < 0 ) { goto gError; }
                                      /* quit exit */
         if ( sLFWN.lh_Head->ln_Succ == NULL ) { iRC = 0; goto gError; }
      }
                                      /* all display wdws */
      if ( *plDisp ) {
         static char cWdwTab[] = {
         "abcdefghijklmnopqrstuvwxyz"
         };
         char *pc = cWdwTab;

         while(*pc) {
            cDispWdw[0] = *pc;
            if ( sFit.eFitWdw( cDispWdw, 1, &sFW, lFlag ) < 0 ) {
               break;
/*
               pMsg = cDispWdw;
               iRC = ERRFIT_NOSUCHWDW; goto gError;
*/
            }
                                      /* duplicate window node */
            if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
               iRC = ERRFIT_SYS; goto gError;
            }
            psFWN->sFW = sFW;
                                      /* link into new list */
            AddTail( &sLFWN, &psFWN->ln ); psFWN = NULL;
            pc++;
         }
      }
                                      /* all conditions */
      if ( *plCond ) {
         long lCnd = 0;

         for(;;) {
            sprintf( cCnd, "%d", ++lCnd );
            if ( sFit.eFitWdw( cCnd, strlen(cCnd), &sFW, lFlag ) < 0 ) {
/*
               pMsg = cCnd;
               iRC = ERRFIT_NOSUCHWDW; goto gError;
*/
               break;
            }
                                      /* duplicate window node */
            if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
               iRC = ERRFIT_SYS; goto gError;
            }
            psFWN->sFW = sFW;
                                      /* link into new list */
            AddTail( &sLFWN, &psFWN->ln ); psFWN = NULL;
         }
      }
   break;
                                      /* windows explicitly specified */
   default:
      if ( ( nWdw = c2vc( (char *) pcWdw, NULL, 1, INT_MAX ) ) <= 0 ) {
         pMsg = (char *) pcWdw;
         iRC = ERRFIT_BADWDWLISTSPEC; goto gError;
      }
      if ( ( pvcWdw = calloc( nWdw, sizeof(char **) ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }
      if ( ( nvcWdw = calloc( nWdw, sizeof(int *) ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }
      if ( c2vcn( (char *) pcWdw, pvcWdw, nvcWdw, 1, nWdw ) != nWdw ) {
         pMsg = (char *) pcWdw;
         iRC = ERRFIT_BADWDWLISTSPEC; goto gError;
      }
                                      /* fetch window limits from caller */
      for ( iWdw=0; iWdw<nWdw; iWdw++ ) {
         memset( &sFW, 0, sizeof(sFW) );
         if ( sFit.eFitWdw( pvcWdw[iWdw], nvcWdw[iWdw], &sFW, lFlag ) < 0 ){
            pMsg = pvcWdw[iWdw];
            iRC = ERRFIT_NOSUCHWDW; goto gError;
         }
                                      /* duplicate window node */
         if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
            iRC = ERRFIT_SYS; goto gError;
         }
         psFWN->sFW = sFW;
                                      /* link into new list */
         AddTail( &sLFWN, &psFWN->ln ); psFWN = NULL;
      }
   break;

   }
                                      /* count # windows */
   nWdw = 0;
   for ( psln=sLFWN.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ) nWdw++;
   if ( nWdw <= 0 ) {
      iRC = ERRFIT_BADWDWCOUNT; goto gError;
   }

/*
   --- Preparations for storing of results
*/
                                      /* set # of passes */
   switch( *plSwitch ) {
   case FITCMDSWITCH_SUM:
                                      /* decide total/sum */
      nPass = MPASS;
      if ( *plTotal ) nPass = FITCMDSUM_TOTAL+1;
      if ( *plNet   ) nPass = FITCMDSUM_NET+1;
                                      /* no data: only total sum possible */
      if ( sFit.sFFB.nf <= 0 ) nPass = FITCMDSUM_TOTAL+1;
                                      /* prepare glopar dimensions */
      nWdw = 0;
      for ( psln=sLFWN.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ) nWdw++;
      nvItem[2] = nItem = MSUMITEM; nvItem[1] = nWdw; nvItem[0] = nPass;

   break;
   case FITCMDSWITCH_LIST:
      nPass = 1;
   break;
   default:
      iRC = ERRFIT_CMDNOTYET; goto gError;
   break;
   }

   if ( *plSwitch == FITCMDSWITCH_SUM ) {
      if ( (dvItem = (double ***) dimalloc( sizeof(double), 3, nvItem )) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }
   }
/*
   --- prepare original data
*/
                                      /* apply global error modes */
   {
            if ( sFit.sFDE.lMode > 0 ) {
               sFit.sFD.sFDE[FIT_DIMY] = sFit.sFDE;
            }

            if ( ( iRC = fitDataErr( &sFit.sFD, FIT_DIMX, lFlag ) ) != 0 )
               goto gError;
            if ( ( iRC = fitDataErr( &sFit.sFD, FIT_DIMY, lFlag ) ) != 0 )
               goto gError;
                                      /* set x-calibration */

            sFit.sFD.sFCalX = sFit.sFCalX;

                                      /* scale for gfx representation */
            if ( iRC = fitDataScale( &sFit.sFD, FIT_DIMX, lFlag | FITDATA_SCALEERR ) )
               goto gError;
                                      /* scale for gfx representation */
            if ( iRC = fitDataScale( &sFit.sFD, FIT_DIMY, lFlag | FITDATA_SCALEERR ) )
               goto gError;
   }
/*
   --- loop total/net
*/
   for ( iPass=0; iPass<nPass; iPass++ ) {

      switch( *plSwitch ) {
                                      /*  */
      case FITCMDSWITCH_SUM:
         sprintf( cStr, "%10s %10s %10s %10s %10s %10s %10s",
                        "Xmin", "Xmax", cSum[iPass], "Error", "Mean",
                        cVar[iMod], cSkw[iMod] );
         _prtcl( cStr, lFlagPrtcl );
         if ( *plGraphic ) {
            if ( iRC = fitTxtXFerPut( &sLTxt, cStr, lFlag ) ) goto gError;
         }
         if ( iPass == FITCMDSUM_NET ) {
                                      /* function values + derivatives */
            if ( ( iRC = fitDataFctBlkDeriv( &sFit.sFD, &sFit.sFFB, lFlag ) ) != 0 ) goto gError;
                                      /* curvature matrices */
            if ( ( iRC = fitDataFctBlkCurv ( &sFit.sFD, &sFit.sFFB, lFlag | lFlagCurve ) ) != 0 ) goto gError;
                                      /* covariance matrix */
            if ( ( iRC = fitFctBlkCovar    (               &sFit.sFFB, lFlag ) ) != 0 ) goto gError;
                                      /* compute function & errors */
            if ( iRC = fitDataFctBlkConf   ( &sFit.sFD, &sFit.sFFB, *pdConf, lFlag ) )
               goto gError;
            if ( iRC = fitDataFctBlkValue  ( &sFit.sFD, &sFit.sFFB,          lFlag ) )
               goto gError;
                                      /*  */
            if ( iRC = fitDataNetScaled( &sFit.sFD, lFlag ) ) goto gError;
            if ( iRC = fitDataScaleFct ( &sFit.sFD, FITDATA_SCALEDIFF, lFlag | FITDATA_SCALEERR ) ) goto gError;
         }
      break;
                                      /*  */
      case FITCMDSWITCH_LIST:
      break;
      default:
      break;
      }

      lFlagPrtcl = CMDPRTCL_FILE | CMDPRTCL_TERM | CMDPRTCL_NOTIME;
                                      /* loop over windows */
      iWdw = 0;
      for ( psln=sLFWN.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ) {

         psFW = &( ( (struct SFITWDWNODE *) psln ) -> sFW );
                                      /* collect data in window */
         memset( &sFD, 0, sizeof(sFD) );
         if ( iRC = fitDataWdw( &sFit.sFD, psFW, &sFD, lFlag ) ) goto gError;
                                      /*  */
         switch( *plSwitch ) {
                                      /*  */
         case FITCMDSWITCH_SUM:
                                      /*  */
            if ( iPass == FITCMDSUM_TOTAL ) lMomAct = FITMOM_DATA;
            else                            lMomAct = FITMOM_DIFF;

            sFW = *psFW;
                                      /* negative sum */
            sFW.dLim[1][0] = -DBL_MAX;
            sFW.dLim[1][1] = -DBL_EPSILON;
            memset( &sFMomNeg, 0, sizeof(sFMomNeg) );
            if ( iRC = fitDataWdwMom( &sFD, &sFW, &sFMomNeg,
                                      lMomAct, lFlag ) ) goto gError;
                                      /* positive sum */
            sFW.dLim[1][1] =  DBL_MAX;
            sFW.dLim[1][0] =  DBL_EPSILON;
            memset( &sFMomPos, 0, sizeof(sFMomPos) );
            if ( iRC = fitDataWdwMom( &sFD, &sFW, &sFMomPos,
                                      lMomAct, lFlag ) ) goto gError;

            if ( *plFWHM ) {
               if ( sFMomPos.dMom[FITMOM_VAR ][FITMOM_VAL] != 0E0 )
                  sFMomPos.dMom[FITMOM_SKEW][FITMOM_VAL] /= sqrt( sFMomPos.dMom[FITMOM_VAR ][FITMOM_VAL] * sFMomPos.dMom[FITMOM_VAR ][FITMOM_VAL] * sFMomPos.dMom[FITMOM_VAR ][FITMOM_VAL]);
               if ( sFMomPos.nn > 1 )
                  sFMomPos.dMom[FITMOM_VAR ][FITMOM_VAL] = sqrt( log(2E0) * 8E0 * (double) sFMomPos.nn / (double) (sFMomPos.nn-1) * sFMomPos.dMom[FITMOM_VAR ][FITMOM_VAL] );
               else sFMomPos.dMom[FITMOM_VAR ][FITMOM_VAL] = 0E0;
            }

            dvItem[iPass][iWdw][FITCMDSUM_XMIN]   = sFW.dLim[0][0];
            dvItem[iPass][iWdw][FITCMDSUM_XMAX]   = sFW.dLim[0][1];
            dvItem[iPass][iWdw][FITCMDSUM_SUM]    = sFMomPos.dMom[FITMOM_SUM ][FITMOM_VAL];
            dvItem[iPass][iWdw][FITCMDSUM_ERRSUM] = sFMomPos.dMom[FITMOM_SUM ][FITMOM_ERRP];
            dvItem[iPass][iWdw][FITCMDSUM_MEAN]   = sFMomPos.dMom[FITMOM_MEAN][FITMOM_VAL];
            dvItem[iPass][iWdw][FITCMDSUM_VAR]    = sFMomPos.dMom[FITMOM_VAR][FITMOM_VAL];
            dvItem[iPass][iWdw][FITCMDSUM_SKEW]   = sFMomPos.dMom[FITMOM_SKEW][FITMOM_VAL];

            sprintf( cStr, "%10.2G %10.2G %10.5G %10.5G %10.5G %10.5G %10.5G",
                           sFW.dLim[0][0], sFW.dLim[0][1],
                           sFMomPos.dMom[FITMOM_SUM ][FITMOM_VAL],
                           sFMomPos.dMom[FITMOM_SUM ][FITMOM_ERRP],
                           sFMomPos.dMom[FITMOM_MEAN][FITMOM_VAL],
                           sFMomPos.dMom[FITMOM_VAR ][FITMOM_VAL],
                           sFMomPos.dMom[FITMOM_SKEW][FITMOM_VAL]
                   );
            _prtcl( cStr, lFlagPrtcl );
            if ( *plGraphic ) {
               if ( iRC = fitTxtXFerPut( &sLTxt, cStr, lFlag ) ) goto gError;
            }
                                      /* output negative sum */
            if ( sFMomNeg.dMom[FITMOM_SUM ][FITMOM_VAL] != 0E0 ) {
               sprintf( cStr, "%10s %10s %10.5G %10s %10s %10s %10s", 
                              " ", " ",
                              sFMomNeg.dMom[FITMOM_SUM ][FITMOM_VAL],
                              " ", " ", " ", " " );
               _prtcl( cStr, lFlagPrtcl );
               if ( *plGraphic ) {
                  if ( iRC = fitTxtXFerPut( &sLTxt, cStr, lFlag ) ) goto gError;
               }
            }
         break; 
                                      /*  */
         case FITCMDSWITCH_LIST:
                                      /* function values + derivatives */
            if ( iRC = fitDataFctBlkDeriv( &sFit.sFDFit, &sFit.sFFB, lFlag ) ) goto gError;
                                      /* curvature matrices */
            if ( iRC = fitDataFctBlkCurv ( &sFit.sFDFit, &sFit.sFFB, lFlag | lFlagCurve ) ) goto gError;
                                      /* covariance matrix */
            if ( iRC = fitFctBlkCovar    (               &sFit.sFFB, lFlag ) ) goto gError;
                                      /* compute function & errors */
            if ( iRC=fitDataFctBlkConf   ( &sFD,         &sFit.sFFB, *pdConf, lFlag ) )
               goto gError;
                                      /* scale for gfx representation */
/*
            if ( iRC=fitDataScaleFct( &sFD, FITDATA_SCALEFCT, lFlag | FITDATA_SCALEERR ) )
               goto gError;
*/

            sprintf( cStr, "%12s %10s %10s %12s %10s %10s %12s %10s %10s",
                           "x", "-dx", "+dx", "y", "-dy", "+dy",
                           "yfit", "-dyfit", "+dyfit"
                   );
            _prtcl( cStr, lFlagPrtcl );
            if ( *plGraphic ) {
               if ( iRC = fitTxtXFerPut( &sLTxt, cStr, lFlag ) ) goto gError;
            }
            for ( ii=0; ii<sFD.nn; ii++ ) {
               sprintf( cStr, "%12G %10G %10G %12G %10G %10G %12G %10G %10G",
                              sFD.pdD[FIT_DIMX][FITDATA_DATA][ii],
                              sFD.pdD[FIT_DIMX][FITDATA_ERRM][ii],
                              sFD.pdD[FIT_DIMX][FITDATA_ERRP][ii],
                              sFD.pdD[FIT_DIMY][FITDATA_DATA][ii],
                              sFD.pdD[FIT_DIMY][FITDATA_ERRM][ii],
                              sFD.pdD[FIT_DIMY][FITDATA_ERRP][ii],
                              sFD.pdFS         [FITDATA_DATA][ii],
                              sFD.pdFS         [FITDATA_ERRM][ii],
                              sFD.pdFS         [FITDATA_ERRP][ii]
                      );
               _prtcl( cStr, lFlagPrtcl );
               if ( *plGraphic ) {
                  if ( iRC = fitTxtXFerPut( &sLTxt, cStr, lFlag ) ) goto gError;
               }
            }
         break; 
         default:
         break;
         }

         fitDataFree( &sFD, lFlag );
         iWdw++;
      } /* wdw */
   } /* pass */

                                      /* */
   if ( *plGraphic ) {
      if ( sFit.eFitGfxTxt( &sLTxt, lFlag ) < 0 ) {
         iRC = ERRFIT_GFXTXTFAILED; goto gError;
      }
   }
                                      /* store into global parameter */

   while ( *pcStore == ' ' ) pcStore++;
   if ( *pcStore && ( *plSwitch == FITCMDSWITCH_SUM ) ) {
      int iDim, nDim, ii, jj, iItem;
      int  nLB[MBTREEDIM], nHB[MBTREEDIM];
      struct SBTREEARRAY sBTA;
      struct SIPAR *pIPar = NULL; double *pd1D; double **pd2D;

      char *pcType, *pcInit, *pcCom;
      char *pcStr = NULL;
      char cStr[80];

                                      /* decompose glopar name */
      if ( ( iRC=btreearr( pcStore, strlen(pcStore), &sBTA ) ) <= 0 ) {
         iRC = ERRFIT_IPARINVNAME; goto gError;
      }
      if ( sBTA.nName <= 0 ) {
         iRC = ERRFIT_IPARINVNAME; goto gError;
      }
      if ( sBTA.lFlag & BTREE_SUBEXPR ) {
         iRC = ERRFIT_IPARINVNAME; goto gError;
      }
      if ( sBTA.lFlag & BTREE_WILD ) {
         iRC = ERRFIT_IPARINVNAME; goto gError;
      }
                                      /* */
      if ( *plNet   ) iPass = FITCMDSUM_NET;
      if ( *plTotal ) iPass = FITCMDSUM_TOTAL;
      if ( iPass >= nPass ) iPass = nPass-1;
                                      /* query target glopar */
      pIPar = _iparptr( pcStore, lFlagI, &iRC );
      switch( iRC ) {
      case 0:                         /* glopar exists */
                                      /*  */
         if ( pIPar -> lFlag & IPAR_READONLY ) {
            iRC = ERRFIT_IPARREADONLY; goto gError;
         }
                                      /* check target data type */
         switch( pIPar->lType ) {
         default:
            iRC = ERRFIT_IPARINVTYP; goto gError;
         break;
         case IPAR_DOUBLE:            /* OK */
         break;
         }
                                      /* check target dimension */
         switch( pIPar->sBTA.nDim ) {
         case 0:
            iRC = ERRFIT_IPARINVDIM; goto gError;
         break;
         case 1:
                                      /* bounds */
                                      /* default */
            nLB[0] = pIPar->sBTA.sDim[0].iLB;
            nHB[0] = pIPar->sBTA.sDim[0].iHB;
                                      /* check user specified bounds */
            switch( sBTA.nDim ) {
            case 0: break;            /* use default */
            case 1:

               if ( sBTA.sDim[0].nLB <= 0 ) {
                  sBTA.sDim[0].iLB = sBTA.sDim[0].iHB;
                  sBTA.sDim[0].iHB = pIPar->sBTA.sDim[0].iHB;
               }
               if ( sBTA.sDim[0].iLB < pIPar->sBTA.sDim[0].iLB ) {
                  iRC = ERRFIT_IPARLOBOUND; goto gError;
               }
               if ( sBTA.sDim[0].iLB > pIPar->sBTA.sDim[0].iHB ) {
                  iRC = ERRFIT_IPARLOBOUND; goto gError;
               }
               if ( sBTA.sDim[0].iHB < pIPar->sBTA.sDim[0].iLB ) {
                  iRC = ERRFIT_IPARHIBOUND; goto gError;
               }
               if ( sBTA.sDim[0].iHB > pIPar->sBTA.sDim[0].iHB ) {
                  iRC = ERRFIT_IPARHIBOUND; goto gError;
               }
               nLB[0] = max( nLB[0], sBTA.sDim[0].iLB );
               nHB[0] = min( nHB[0], sBTA.sDim[0].iHB );
            break;
            default:
               iRC = ERRFIT_IPARINVDIM1; goto gError;
            break;
            }

            nHB[0] = min( nLB[0] + nWdw - 1, pIPar->sBTA.sDim[0].iHB );

if ( lFlag & FIT_DEBUG )
fprintf( stderr, "<D> %d %d\n    %d %d\n    %d %d\n",
                 sBTA.sDim[0].iLB, sBTA.sDim[0].iHB,
                 pIPar->sBTA.sDim[0].iLB, pIPar->sBTA.sDim[0].iHB,
                 nLB[0], nHB[0] );

            if ( (nHB[0] - nLB[0] + 1) <= 0 ) {
               iRC = ERRFIT_IPARRANGEZERO; goto gError;
            }
                                      /* copy data */
            pd1D = (double *) pIPar->pv; iWdw = 0;
            for ( ii=nLB[0]; ii<=nHB[0]; ii++ ) {
               pd1D[ii] = dvItem[iPass][iWdw++][FITCMDSUM_SUM];
            }
         break; /* target 1D */

         case 2:
                                      /* default */
            for ( iDim=0; iDim<pIPar->sBTA.nDim; iDim++ ) {
               nLB[iDim] = pIPar->sBTA.sDim[iDim].iLB;
               nHB[iDim] = pIPar->sBTA.sDim[iDim].iHB;
            }
                                      /* check user specified bounds */
            switch( sBTA.nDim ) {
            case 0: break;            /* use default */
            case 2:

               for ( iDim=0; iDim<sBTA.nDim; iDim++ ) {
                  if ( sBTA.sDim[iDim].nLB <= 0 ) {
                     sBTA.sDim[iDim].iLB = sBTA.sDim[iDim].iHB;
                     sBTA.sDim[iDim].iHB = pIPar->sBTA.sDim[iDim].iHB;
                  }
                  if ( sBTA.sDim[iDim].iLB < pIPar->sBTA.sDim[iDim].iLB ) {
                     iRC = ERRFIT_IPARLOBOUND; goto gError;
                  }
                  if ( sBTA.sDim[iDim].iLB > pIPar->sBTA.sDim[iDim].iHB ) {
                     iRC = ERRFIT_IPARLOBOUND; goto gError;
                  }
                  if ( sBTA.sDim[iDim].iHB < pIPar->sBTA.sDim[iDim].iLB ) {
                     iRC = ERRFIT_IPARHIBOUND; goto gError;
                  }
                  if ( sBTA.sDim[iDim].iHB > pIPar->sBTA.sDim[iDim].iHB ) {
                     iRC = ERRFIT_IPARHIBOUND; goto gError;
                  }
                  nLB[iDim] = max( nLB[iDim], sBTA.sDim[iDim].iLB );
                  nHB[iDim] = min( nHB[iDim], sBTA.sDim[iDim].iHB );
               }
            break;
            default:
               iRC = ERRFIT_IPARINVDIM2; goto gError;
            break;
            }

            for ( iDim=0; iDim<pIPar->sBTA.nDim; iDim++ ) {
               nHB[iDim] = min( nLB[iDim] + nvItem[iDim+1] - 1, pIPar->sBTA.sDim[iDim].iHB );
               if ( (nHB[iDim] - nLB[iDim] + 1) <= 0 ) {
                  iRC = ERRFIT_IPARRANGEZERO; goto gError;
               }
            }
                                      /* copy data */
            pd2D = (double **) pIPar->pv; iWdw = 0;
            for ( ii=nLB[0]; ii<=nHB[0]; ii++ ) {
               iItem = 0;
               for ( jj=nLB[1]; jj<=nHB[1]; jj++ ) {
                  pd2D[ii][jj] = dvItem[iPass][iWdw][iItem++];
               }
               iWdw++;
            }
                                      /* default bounds */
         break; /* target dim 2D */

         default:                     /* dim > 2 */
         break;
         }
      break;
      case ERRCMD_IPARNOTFOUND:       /* glopar does not exist */
                                      /* build name */
         sprintf( cStr, "(%d,%d)", nWdw, nItem );
         if ( ( pcStr = strdup2( pcStore, cStr ) ) == NULL ) {
            iRC = ERRFIT_SYS; goto gError;
         }
                                      /* create glopar */
         pIPar = _ipardef( pcStr, pcType="double", pcInit="", pcCom="fsum",
                           lFlag | IPAR_DYNAMIC, &iRC );
         free(pcStr); pcStr = NULL;
         if ( ( iRC != 0 ) || ( pIPar == NULL ) ) {
            iRC = ERRFIT_IPARCREATEFAILED; goto gError;
         }
                                      /* store values */
         dimcopy( (void **) dvItem[iPass], pIPar->pv,
                  sizeof(double), nDim=2, nvItem+1 );
      break;
      default:
         iRC = ERRFIT_IPARINVNAME; goto gError;
      break;
      }
/*
fprintf( stderr, "<D> name=%*.*s\n", sBTA.nName, sBTA.nName, sBTA.cName );
fprintf( stderr, "<D> dim=%d\n", sBTA.nDim );
for( iDim=0; iDim<sBTA.nDim; iDim++ ) {
   fprintf( stderr, "<D> %d %d %d %d\n", iDim,
                    sBTA.sDim[iDim].iLB,
                    sBTA.sDim[iDim].iHB,
                    sBTA.sDim[iDim].iStep
          );
}
*/
   }

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
                                      /* cleanup */
   dimfree( (void **) dvItem, 3, nvItem );
   fitDataFree( &sFD, lFlag );
   if ( psFWN  ) free( psFWN  ); psFWN = NULL;
   if ( nvcWdw ) free( nvcWdw ); nvcWdw = NULL;
   if ( pvcWdw ) free( pvcWdw ); pvcWdw = NULL;
   fitWdwListFree( &sLFWN, lFlag );
   fitTxtXFerFree( &sLTxt, lFlag );
                   
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return;
}
