#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <limits.h>

#include "u.h"
#include "m.h"
#include "cmd.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitcmdwdw.c,v 1.4 1997/12/15 18:57:24 kraemer Exp $"};

extern struct SFIT sFit;
/*
   --- Command processor for the FWINDOWS command

   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitcmdwdw.c,v $
   Revision 1.4  1997/12/15 18:57:24  kraemer
   gfx flush added

   Revision 1.3  1997/08/29 12:26:16  kraemer
   *** empty log message ***

   Revision 1.2  1997/08/29 12:10:29  kraemer
   gfxattr rework

   Revision 1.1.1.1  1997/07/10 18:00:02  kraemer
   revision for cvs

*/

void fitCmdWdw(
                const char *pcWdw,
                long *plAll, long *plDisp, long *plCond,
                long *plLoop, long *plKeep, long *plList, long *plMark,
                long *plDebug
              ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__, *pMsg = pDbgMsg;
   unsigned long lFlag = 0;

   struct SFITGFX sFGfx;
   struct SFITWDW sFW, sFWDef, *psFW;
   struct SFITWDWNODE *psFWN = NULL;
   struct List sLFW, *psLFW = NULL;
   struct Node *psln;

   int ii, iDim;
   int iWdw,nWdw;
   int *nvcWdw = NULL;
   char **pvcWdw = NULL;

   char cDispWdw[2] = { 0, 0 };
   char cCnd[20]= {""};


   if ( *plDebug ) lFlag |= FIT_DEBUG;
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );

   NewList( &sLFW );
   memset( &sFGfx, 0, sizeof(sFGfx) );
                                      /* check valid inputs */
   if ( ( *plLoop + *plDisp + *plCond + *plAll ) > 1 ) {
      pMsg = "loop/disp/cond/all";
      iRC = ERRFIT_CMDPAREXCL; goto gError;
   }
                                      /* check window interface exists */
   if ( sFit.eFitWdw == NULL ) {
      iRC = ERRFIT_NOWDWAVAIL; goto gError;
   }
                                      /* get display limits */
   if ( sFit.eFitWdw( "#", 1, &sFWDef, lFlag ) < 0 ) {
      pMsg = "#";
      iRC = ERRFIT_NOSUCHWDW; goto gError;
   }

   sFGfx.pGH = _GFndWdw( sFit.pcWdw );
   if ( lFlag & FIT_DEBUG ) sFGfx.lFlag |= G_DEBUG;

   if ( *plAll ) *plKeep = 0;         /* if ALL specified,
                                            KEEP does not make sense */
   psLFW = &sLFW;                     /* new list of windows */
   if ( *plKeep ) psLFW = &sFit.sListFW; /* old list */
                                      /* skip blanks */
   while( *pcWdw == ' ' ) pcWdw++;
   if ( *pcWdw ) {                    /* windows specified */
      if ( ( nWdw = c2vc( (char *) pcWdw, NULL, 1, INT_MAX ) ) <= 0 ) {
         pMsg = (char *) pcWdw;
         iRC = ERRFIT_BADWDWLISTSPEC; goto gError;
      }
      if ( ( pvcWdw = calloc( nWdw, sizeof(char **) ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }
      if ( ( nvcWdw = calloc( nWdw, sizeof(int *) ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      }
      if ( c2vcn( (char *) pcWdw, pvcWdw, nvcWdw, 1, nWdw ) != nWdw ) {
         pMsg = (char *) pcWdw;
         iRC = ERRFIT_BADWDWLISTSPEC; goto gError;
      }
                                      /* fetch window limits from caller */
      for ( iWdw=0; iWdw<nWdw; iWdw++ ) {
         memset( &sFW, 0, sizeof(sFW) );
         if ( sFit.eFitWdw( pvcWdw[iWdw], nvcWdw[iWdw], &sFW, lFlag ) < 0 ){
            pMsg = pvcWdw[iWdw];
            iRC = ERRFIT_NOSUCHWDW; goto gError;
         }
                                      /* duplicate window node */
         if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
            iRC = ERRFIT_SYS; goto gError;
         }
         psFWN->sFW = sFW;
                                      /* link into new list */
         AddTail( psLFW, &psFWN->ln ); psFWN = NULL;
      }
   } else {
                                      /* nothing specified: loop is default */
      if ( ( (*plAll + *plDisp + *plCond + *plLoop) == 0 ) &&
           ( (*plMark + *plList) == 0 ) )
         *plLoop =1;
                                      /* prompt window by cursor/terminal */
      if ( *plLoop ) {
         sFW = sFWDef;                /* default = display limit */
         while( (iRC = fitWdwPrompt( &sFW, &sFGfx, lFlag )) == 0 ) {
                                      /* duplicate window node */
            if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
               iRC = ERRFIT_SYS; goto gError;
            }
            psFWN->sFW = sFW;
                                      /* link into new list */
            AddTail( psLFW, &psFWN->ln ); psFWN = NULL;
            sFW = sFWDef;
         }
         if ( iRC < 0 ) goto gError;
      }
                                      /* open limits */
      if ( *plAll ) {
                                      /* delete all windows */
          fitWdwListFree( &sFit.sListFW, lFlag );
#if defined(REALLY)
         if ( sFit.eFitWdw( "*", 1, &sFW, lFlag ) < 0 ) {
            pMsg = "*";
            iRC = ERRFIT_NOSUCHWDW; goto gError;
         }
                                      /* duplicate window node */
         if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
            iRC = ERRFIT_SYS; goto gError;
         }
         psFWN->sFW = sFW;
                                      /* link into new list */
         AddTail( psLFW, &psFWN->ln ); psFWN = NULL;
#endif
      }
                                      /* all display wdws */
      if ( *plDisp ) {
         static char cWdwTab[] = {
         "abcdefghijklmnopqrstuvwxyz"
         };
         char *pc = cWdwTab;

         while(*pc) {
            cDispWdw[0] = *pc;
            if ( sFit.eFitWdw( cDispWdw, 1, &sFW, lFlag ) < 0 ) {
               break;
/*
               pMsg = cDispWdw;
               iRC = ERRFIT_NOSUCHWDW; goto gError;
*/
            }
                                      /* duplicate window node */
            if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
               iRC = ERRFIT_SYS; goto gError;
            }
            psFWN->sFW = sFW;
                                      /* link into new list */
            AddTail( psLFW, &psFWN->ln ); psFWN = NULL;
            pc++;
         }
      }
                                      /* all conditions */
      if ( *plCond ) {
         long lCnd = 0;

         for(;;) {
            sprintf( cCnd, "%d", ++lCnd );
            if ( sFit.eFitWdw( cCnd, strlen(cCnd), &sFW, lFlag ) < 0 ) {
/*
               pMsg = cCnd;
               iRC = ERRFIT_NOSUCHWDW; goto gError;
*/
               break;
            }
                                      /* duplicate window node */
            if ( ( psFWN = calloc( 1, sizeof(*psFWN) ) ) == NULL ) {
               iRC = ERRFIT_SYS; goto gError;
            }
            psFWN->sFW = sFW;
                                      /* link into new list */
            AddTail( psLFW, &psFWN->ln ); psFWN = NULL;
         }
      }
   }
                                      /*  */
   if ( psLFW == &sLFW ) {            /* new list */
                                      /* if non-empty:
                                         delete old, take new list */
      if ( psLFW -> lh_Head -> ln_Succ ) {
         fitWdwListFree( &sFit.sListFW, lFlag );
         while( sLFW.lh_Head -> ln_Succ ) {
            psln = RemHead( &sLFW );
            AddTail( &sFit.sListFW, psln );
         }
      }
   }
                                      /* list windows */
   if ( *plList ) {
      unsigned long lFlagPrtcl = CMDPRTCL_TERM | CMDPRTCL_FILE;
      int iWdw=0;
      char cStr[8+MFITWDWDIM*16*2+1] = {""};

      for( psln = sFit.sListFW.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ) {
         psFW = &( (struct SFITWDWNODE *) psln ) -> sFW;
         sprintf( cStr, "%6d", ++iWdw );
         for ( iDim=0; iDim<psFW->nDim; iDim++ ) {
            sprintf( cStr, "%s %12G %12G", cStr,
                            psFW->dLim[iDim][0],
                            psFW->dLim[iDim][1]
                   );
         }
         _prtcl( cStr, lFlagPrtcl ); lFlagPrtcl |= CMDPRTCL_NOTIME;
      }
   }
                                      /* mark windows */
   if ( *plMark && sFGfx.pGH ) {
      double dXY[2][2] = { -DBL_MAX, DBL_MAX, -DBL_MAX, DBL_MAX };
      struct SGFXATTR sGA;
                                      /* get display limits */
      if ( sFit.eFitWdw( "#", 1, &sFW, lFlag ) < 0 ) {
         pMsg = "#";
         iRC = ERRFIT_NOSUCHWDW; goto gError;
      }
      dXY[0][1] = sFW.dLim[1][0];
      dXY[1][1] = sFW.dLim[1][1];

      for( psln = sFit.sListFW.lh_Head; psln->ln_Succ; psln=psln->ln_Succ ) {
         psFW = &( (struct SFITWDWNODE *) psln ) -> sFW;
                                      /* lower/upper limit */
         for ( ii=0; ii<2; ii++ ) {
            dXY[0][0] = psFW->dLim[0][ii];
            dXY[1][0] = psFW->dLim[0][ii];
            sGA = sFit.sGA;
            sGA.dLWidth = 0E0;
            sGA.lLMode = GFXATTR_LINEDASH;
            _GGfxAttrSet( sFGfx.pGH, &sGA,
                          GGFXATTR_COLOUR | GGFXATTR_FCT |
                          GGFXATTR_LMODE | GGFXATTR_LWIDTH | sFGfx.lFlag );
            _GLine  ( sFGfx.pGH, dXY, 2, sFGfx.lFlag | G_COORDLOG );
         }
      }
      _GFlush( sFGfx.pGH, sFGfx.lFlag );
   }

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
                                      /* cleanup */
   fitWdwListFree( &sLFW, lFlag );
   if ( psFWN ) free( psFWN ); psFWN = NULL;
   if ( pvcWdw ) free( pvcWdw ); pvcWdw = NULL;
   if ( nvcWdw ) free( nvcWdw ); nvcWdw = NULL;

if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return;
}

