#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>

#include "u.h"
#include "m.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fitstat.c,v 1.2 1997/12/15 19:02:13 kraemer Exp $"};

/*
   --- Functions to handle fit statistical and miscellaneous info

       fitStatProb2Chi(): compute chi**2 from probability
       fitStatChi2Prob(): compute probability from chi**2
       fitStatInit()    : initialize
       fitStatInspect() : inspect output
 
   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fitstat.c,v $
   Revision 1.2  1997/12/15 19:02:13  kraemer
   obsolete chiprob logic

   Revision 1.1.1.1  1997/07/10 18:00:07  kraemer
   revision for cvs

*/
/*
   =========================================================================
*/
int fitStatChi2Prob( struct SFITSTAT *psFS,
                     unsigned long lFlag
                   ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitStatChi2Prob)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFS == NULL ) goto gError;

   if ( psFS->nDegFree > 0 )
/*
      psFS->dProb = chiprob( psFS->dChisq / psFS->nDegFree, psFS->nDegFree);
*/
      psFS->dProb = chiProb( psFS->dChisq, psFS->nDegFree);

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitStatProb2Chi( struct SFITSTAT *psFS,
                     unsigned long lFlag
                   ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitStatProb2Chi)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFS == NULL ) goto gError;
   if ( psFS->nDegFree <= 0 ) { iRC = ERRFIT_STATNDF; goto gError; }

   psFS -> dChisq = chiQuantil( psFS->dProb, psFS->nDegFree );

gHome:;
   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitStatInit( struct SFITSTAT *psFS,
                 unsigned long lFlag
               ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitStatInit)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFS == NULL ) goto gError;

   memset( psFS, 0, sizeof(*psFS) );
   psFS -> dEps[0] = 1E-1;
   psFS -> dEps[1] = 1E-1;
   psFS -> dEps[2] = 1E-5;
   psFS -> dEps[3] = 5E-1;
   psFS -> dTau    = 1E-7;
   psFS -> lL0     = 0;
   psFS -> lLd     = 2;
   psFS -> lEIter  = 100 /*4*/;
   psFS -> dLambda = 1E-3;
   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitStatInspect( struct SFITSTAT *psFS,
                    unsigned long lFlag
                  ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitStatInspect)", *pMsg = pDbgMsg;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( psFS == NULL ) goto gError;

   fprintf( stderr, "<D> SFITSTAT=%X\n", psFS );
   fprintf( stderr, "<D> status=%X\n", psFS->lStatus );
   fprintf( stderr, "<D> ndf=%X\n", psFS->nDegFree );
   fprintf( stderr, "<D> chisq=%G loglike=%G chiprob=%G\n",
                         psFS->dChisq, psFS->dLogLike, psFS->dProb );
   fprintf( stderr, "<D> eps1=%G eps2=%G eps3=%G eps4=%G\n",
                         psFS->dEps[0], psFS->dEps[1], psFS->dEps[2], psFS->dEps[3]);
   fprintf( stderr, "<D> tau=%G lambda=%G\n",
                         psFS->dTau, psFS->dLambda );
   fprintf( stderr, "<D> l0=%ld ld=%ld eiter=%ld\n",
                         psFS->lL0, psFS->lLd, psFS->lEIter );
   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
