#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>

#include "u.h"
#include "m.h"
#include "g.h"
#include "fit.h"

static char rcsid[] = {"$Id: fittxtxfer.c,v 1.1.1.1 1997/07/10 18:00:07 kraemer Exp $"};

/*
   --- Functions to handle transfer of text for graphical output

       fitTxtXFerPut()     : insert a new text 
       fitTxtXFerFree()    : frees resources
       fitTxtXFerInspect() : inspects contents
 
   Copyright  (C) 1987-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved
   $Log: fittxtxfer.c,v $
   Revision 1.1.1.1  1997/07/10 18:00:07  kraemer
   revision for cvs

*/

int fitTxtXFerPut(
                    struct List *pslh,
                    const char *pcTxt,
                    unsigned long lFlag
                   ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitTxtXFerPut)", *pMsg = pDbgMsg;
   struct Node *psln = NULL; 
   char *pc = (char *) pcTxt; int ncTxt;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( pslh == NULL ) goto gError;
   if ( pcTxt == NULL ) goto gError;
                                      /*  */
   while( *pcTxt ) {
      ncTxt = strlen( pcTxt );
      if ( pc = strchr( pcTxt, '\n' ) )
         ncTxt = (unsigned long) pc - (unsigned long) pcTxt + 1;
      if ( ( psln = calloc( 1, sizeof(*psln) ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      } 
      if ( ( psln->ln_Name = strndup( pcTxt, ncTxt ) ) == NULL ) {
         iRC = ERRFIT_SYS; goto gError;
      } 
      AddTail( pslh, psln ); psln = NULL;
      pcTxt += ncTxt;
   }

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }

   if ( psln ) {
      if ( psln->ln_Name ) free( psln->ln_Name ); psln->ln_Name = NULL;
      free(psln); psln = NULL;  
   }
if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitTxtXFerFree(
                    struct List *pslh,
                    unsigned long lFlag
                   ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitTxtXFerFree)", *pMsg = pDbgMsg;
   struct Node *psln;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( pslh == NULL ) goto gError;
                                      /* free */
   while( pslh -> lh_Head -> ln_Succ ) {
      psln = RemTail( pslh );
      if ( psln -> ln_Name ) free( psln -> ln_Name ); psln -> ln_Name = NULL;
      free( psln ); psln = NULL;
   }

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }

if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/
int fitTxtXFerInspect(
                    struct List *pslh,
                    unsigned long lFlag
                   ) {
   int iRC = ERRFIT_BADADDR;
   char *pDbgMsg = __FILE__"(fitTxtXFerInspect)", *pMsg = pDbgMsg;
   struct Node *psln;


if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s.....\n", pDbgMsg );
                                      /* check valid inputs */
   if ( pslh == NULL ) goto gError;
                                      /* */
   fprintf( stderr, "<D> SFITTXTXFER=%X\n", pslh );
   for( psln=pslh->lh_Head; psln->ln_Succ; psln = psln->ln_Succ ) {
      if ( psln -> ln_Name ) fprintf( stderr, "<D> %s\n", psln -> ln_Name ); 
   } 

   iRC = 0;
gError:;
   if ( (iRC < 0) && !(lFlag&FIT_NOMSG) ) {
      fitemsg( pMsg, iRC );
   }

if ( lFlag & FIT_DEBUG ) fprintf( stderr, "<D> %s done\n", pDbgMsg );
   return(iRC);
}
/*
   =========================================================================
*/

