/* *****************************************************************

   File:        atmdf1_shm.h
   Description: Data exchange and program control 
                via shared memory
   Revision:    0.01 (28 Jan 2002)
                0.02 (30 Jan 2002) Added Program control
		0.03 (31 Jan 2002) Bugfixes F1ShmGetBufferAddr
		0.04 (05 Feb 2002) Seperated dontrol and Datasegment
		**************************************************** */


#ifndef ATMDF1SHM_INC
#define ATMDF1SHM_INC


#include <sys/time.h>

#define PATHTODATA "/tmp/shmdata"
#define PATHTOCTRL "/tmp/shmctrl"

#define SHM_KEY_DATA  (ftok (PATHTODATA, 'C'))
#define SHM_KEY_CTRL  (ftok (PATHTOCTRL, 'C'))
#define SHM_MODE (0660)
#define BUF_CTRLCMD  (128)

/*
  Parameters including channel numbers are handled as 
  unsigned long (32 bit) integers
*/
typedef unsigned long data_t;

/* Parameter and channel numbers are unsigned 16 Bit integers */
typedef unsigned short data_p;
typedef unsigned short data_ch;

/* A block is a sequence of parameters or events with a sequence header */
/* This structur definition does not contain the data itself.           */

typedef struct block_t{
  struct timeval ts;                           /* Timestamp             */
  long bnext;                                  /* Pointer to next block */
  unsigned long n;                             /* Parameter count       */
  unsigned long seq;                           /* Sequence number       */
} block_t;


/* A buffer contains this header information followed by blocks                   */

typedef struct buffer_t{
  unsigned long ptwrite;     /* Write pointer, used by shm writer to store blocks */
  unsigned long ptread;      /* Read pointer, used by shm reader                  */
  char ctrlcmd[BUF_CTRLCMD]; /* Control command buffer                            */
  int status;                /* Status: 0: idle, 1: command busy, 2: Accept       */
  int daq;
  int rt;                    /* return value                                      */
  pid_t pid;                 /* Process ID of the daemon                          */
  int parameter;             /* parameter per event                               */
  char hits[8];              /* Hits per channel                                  */
  int divider;               /* Event Divider                                     */
  int shmid_data;
  int channelmask;
  int error;                 /* Global error flag                                 */
  int filemode;              /* BIT16FILEMODE or BIT32FILEMODE                    */
} buffer_t;

#define EVENTS_MAX 7000

#define F1EVENTSORT  1
#define F1COMMANDRUN 2

#endif



