/* *****************************************************

   File:         atmdf1lib.h                           
   Description:  Routines inside libatmdf1.a
                 Basic constants and structures
   Revision:     0.01 (28 Feb 2002)

   ***************************************************** */

#ifndef ATMDF1LIB_INC
#define ATMDF1LIB_INC

#include "atmdf1_lib.h"
#include "atmdf1_shm.h"
#include <time.h>

#define ATMDF1DEV "/dev/acam"
#define TDC_BUFFER_SIZE      100
#define LISTMODE_BUFFER_SIZE 512
#define FIFO_SIZE          65536

#define ASCII_MODE 1
#define LIST_MODE  2
#define SHMEM_MODE 4
#define RAW_MODE   8

#define BIT16FILEMODE 0
#define BIT32FILEMODE 1

#define FALSE 0
#ifndef TRUE
#define TRUE !FALSE
#endif
#define CHANNELS 8

#define DAEMON_IDLE   (0)
#define DAEMON_ACCEPT (2)
#define DAEMON_BUSY   (1)
#define CMDTIMEOUT    (1000)


#define ERR_GENERIC      (-1)
#define ERR_NODRIVER     (-2)
#define ERR_NOSHMCTRLSEG (-3)
#define ERR_NODAEMON     (-4)
#define ERR_NOSHMDATASEG (-5)
#define ERR_DAEMONMODE   (-6)
#define ERR_DAEMONPROC   (-7)
#define ERR_SHMCTRLSEGPRESENT  (-8)
#define ERR_PARMOUTOFRANGE (-9)
#define ERR_FAILCREATEDATASEG (-10)
#define ERR_DAQRUNNING  (-11)
#define ERR_PERMDENIED  (-12)
#define ERR_DAEMONBUSY  (-13)
#define ERR_NOLMFSPECIFIED (-14)
#define ERR_ARGCOUNTMISMATCH (-15)
#define ERR_CMDUNKNOWN       (-16)
#define ERR_SPECDATASOURCE   (-17)
#define ERR_STATUS_DAQRUNNING (-18)
#define ERR_DISKFULL          (-19)

#if defined(__linux__)
#define PRINT(fs, da...) F1log (NULL, fs, da)
#define INFO_PRINT(fs, da... ) F1log ("Info", fs, ##da)
#define WARN_PRINT(fs, da... ) F1log ("Warning", fs, ##da)
#define ERROR_PRINT(fs, da... ) F1log ("Error", fs, ##da)
#else
#define PRINT(fs, da ) F1log (NULL, fs, da)
#define INFO_PRINT(fs, da ) F1log ("Info", fs, da)
#define WARN_PRINT(fs, da ) F1log ("Warning", fs, da)
#define ERROR_PRINT(fs, da ) F1log ("Error", fs, da)
#endif

typedef struct event{
  char hits[CHANNELS];
  int parameter;
} event;


typedef struct ATMDF1 {
  unsigned int error;
  event e;              /* Event structur                        */
  unsigned long divider;
  int channelmask;
  int datamode;
  int atmd;             /* File handle to atmdf1 driver          */
  char* data_segment;   /* Pointer to shared memory data segment */
  buffer_t* ctrl_segment;   /* Pointer to shared memory ctrl segment */
  long shm_data_size;   /* Size of shm data segment              */
  int shmid_data;
  int shmid_ctrl;
  char listmode_path[512];
  char rawmode_path[512];
  FILE *listmode_fd;
  FILE *rawmode_fd;
  FILE *source_fd;
  data_t *event_buffer; /* Pointer to event buffer dynamically allocated */
  block_t *shm_lastblock;
  unsigned int sep_low;
  unsigned int sep_high;
  int sep_pass;
  int filemode;
} ATMDF1;



         int  F1Sleep            (int usec);
unsigned int  F1CommonMode       (int atmd, int refclkdiv, int hsdiv, int phase, int hres);
unsigned int  F1ChannelsEnable   (int atmd, int channels);
unsigned int  F1ChannelsDisable  (int atmd, int channels);
unsigned int  F1Edge             (int atmd, int channel, int edge );
unsigned int  F1Threshold        (int atmd, int channel, int val);
void          F1ThresholdsEnable (int atmd);
unsigned int  F1InitUnits        (int atmd);
void          F1EventSet         (event *e);
void          F1DividerSet       (ATMDF1 *atmd, int div);
unsigned int  F1Reset            (ATMDF1 *atmd);
unsigned int  F1Burstmode        (ATMDF1 *atmd, int enable);
int           F1ListmodeOpen     (ATMDF1 *atmd, char *path);
int           F1RawmodeOpen      (ATMDF1 *atmd, char *path);
void          F1ListmodeClose    ();
void          F1RawmodeClose     ();
int           F1RawWrite         (ATMDF1 *atmd, unsigned long seq, unsigned int n);
int           F1DatamodeSet      (ATMDF1 *atmd, int mode);
int           F1DatamodeClr      (ATMDF1 *atmd, int mode);
unsigned int  F1Read             (ATMDF1 *atmd, unsigned int count);
unsigned long F1Event            (ATMDF1 *atmd, struct timeval tm, int sequence, unsigned int nreed);
void          F1LibOpen          (ATMDF1 *atmd);
void          F1LibClose         (ATMDF1 *atmd);
unsigned long F1ShmRead   (ATMDF1 *atmd, data_t *dt, unsigned long *sequence, struct timeval *tm);
unsigned long F1ShmRead16 (ATMDF1 *atmd, data_t *dt, unsigned long *sequence, struct timeval *tm);
void          F1ShmDetachReader (ATMDF1 *atmd, data_t *data);
void          F1ShmDetachReaderCtrl (ATMDF1 *atmd);
int           F1ShmAttachReaderData (ATMDF1 *atmd, data_t **data);
int           F1ShmAttachReaderCtrl (ATMDF1 *atmd);
void          F1ShmWrite   (ATMDF1 *atmd, struct timeval *tm, long sequence, long n);
void          F1ShmWrite16 (ATMDF1 *atmd, struct timeval *tm, long sequence, long n);
int           F1ShmCheck   (ATMDF1 *atmd);
int           F1ShmCheck16 (ATMDF1 *atmd);
void          F1ShmInit (ATMDF1 *atmd);
void          F1ShmDetachCtrlSegement(ATMDF1 *atmd);
buffer_t*     F1ShmAttachCtrlSegement (ATMDF1 *atmd);
int           F1ShmDestroySegments (ATMDF1 *atmd);
int           F1ShmRemoveDataSegment (ATMDF1 *atmd);
int           F1ShmCreateCtrlSegment (ATMDF1 *atmd);
int           F1ShmCreateDataSegment (ATMDF1* atmd);
void          F1BufferFree (ATMDF1 *atmd);
void          F1BufferAlloc (ATMDF1 *atmd);
int           F1SendCommand (ATMDF1 *atmd, char *cmd);
int           F1log (char *spec, char *fs, ...);
int           F1DriverOpen (ATMDF1 *atmd);
void          F1DriverClose (ATMDF1 *atmd);
#endif

