#ifndef CMDAREAD_H
/*

   Definitons to handle data in GSI's "AREAD" format.

   Copyright  2000 M.Kraemer, GSI, D-64291 Darmstadt

   $Log: cmdaread.h,v $
   Revision 1.1  2000/04/03 13:10:24  kraemer
   *** empty log message ***


*/
#define CMDAREAD_H

#include "cmderr.h"

                                      /* global flags */
#define CMDAREAD_DEBUG  0x80000000    /* general check flag */
#define CMDAREAD_QUIET  0x10000000    /* suppress error messages */
#define CMDAREAD_NOMSG  CMDAREAD_QUIET

#define CMDAREAD_MAXDIM 4
                                      /* structure to store data from file */
struct SCMDAREAD {
   unsigned long ulFlag;              /* hold general flag bits */
   unsigned long ulSpace;
   char *pcAnl, *pcEvt;               /* analyzer & event name */
   unsigned long ulType;              /* spectrum data type */
   int iSizeOf;
   long lDim;                         /* dimension */
   unsigned long ulData;              /* # of data items */
   long   lvLim[CMDAREAD_MAXDIM][4]; 
   double dvLim[CMDAREAD_MAXDIM][4]; 
   void *pv;                          /* [ulData] */
   short *pi;  
   long  *pl;
   float *pr;                     
   struct SCMDERR sCE;
};

enum {
   CMDAREAD_TYPESHORT=2, CMDAREAD_TYPELONG=4, CMDAREAD_TYPEFLOAT=24,
};

/*

   Functions
   ---------
   
   cmdAReadFree    : ( aread, flag )
                   3 x to test
                     o calls:
                       o cmdAReadInit()
                            
   cmdAReadInit    : ( aread, flag )
                   2 x to test
                     o calls:
                         
   cmdAReadInspect : ( aread, flag )
                   1 x to test
                     o calls:
                         
   cmdAReadRead    : ( aread, map, flag )
                   4 x to test
                     ? determine data type from data format, if no type given
                     o calls:
                       o cmdAReadInit     
                       o cmdAReadFree     

*/
                                      /* function prototypes */
int cmdAReadFree   ( struct SCMDAREAD *, unsigned long );
int cmdAReadInit   ( struct SCMDAREAD *, unsigned long );
int cmdAReadInspect( struct SCMDAREAD *, unsigned long );
int cmdAReadRead   ( struct SCMDAREAD *, const char *, unsigned long );

struct SCMDAREAD *_cmdAReadTest( unsigned long );

                                      /* error handling */
int cmdAReadError( struct SCMDERR *, unsigned long );

enum { ERRCMDAREAD_SYS=1,
       ERRCMDAREAD_BADADDR,
       ERRCMDAREAD_BADSYNTAX,
       ERRCMDAREAD_BADNUMBER, 
       ERRCMDAREAD_BADDIM,
       ERRCMDAREAD_BADDIMINDEX,
       ERRCMDAREAD_BADTYPE, 
       ERRCMDAREAD_BADCHANNEL, 
       ERRCMDAREAD_BADBINSIZE, 
       ERRCMDAREAD_BADNDATA, 
       ERRCMDAREAD_DATAINCONSISTENT, 
};

#endif

