#ifndef IOEDAS_H
#define IOEDAS_H
/*
   Definitions for I/O of EDAS data 
   =================================

   Copyright  (C) 1995-2000 M.Kraemer, GSI Darmstadt 
                             all rights reserved   

   $Log: ioedas.h,v $
   Revision 1.1  1999/07/17 16:18:49  kraemer
   major overhaul of edas/goosy buffer and swap logic


*/


#include "u.h"
#include "iofh.h"

#define IO_SYSSWAP   0x00000080       /* data swap needed */

                                      /* buffer element header */
struct SIOEDASBEH {
   long lDLen;                        /* length of buffer in 16bit words,
                                         without this header */ 
#if defined(U_BIG_ENDIAN)
   short iSubType, iType;             /* buffer subtype/type */ 
#elif defined(U_LITTLE_ENDIAN)
   short iType, iSubType;             /* buffer subtype/type */
#else
#error you loose, unknown endian
#endif
};

                                      /* datablock header */
struct SIOEDASDBH {
   short nData;                       /* # of data (bytes) following this header   */
   unsigned bfSpare : 3;              /* not used */ 
   unsigned bfVo : 1;                 /* VSAM bit, if set: data are reorganized */
   unsigned bfVc : 1;                 /* VSAM bit, if set: data are compressed */
   unsigned bfNFB : 1;                /* non-forgettable bit,
                                         if not set, the online data analysis
					 may skip this record */
   unsigned char ucDataType;          /* data type */
   unsigned char ucSpare;
   unsigned char ucSubType;           /* subtype, for data type=7 only */
   unsigned char ucBmax;              /* max # of correlated records (same data type) */					 
   unsigned char ucBcur;              /* data block number for correlated records (same data type) */					 
   long lBlock;                       /* current block number, starting with 0 in each file */ 
};				      				      				      
                                      /* EDAS data types */
enum {
   IOEDAS_FILEHEAD=0,                 /* file header, subtype=0 */
   IOEDAS_ANL=1,                      /* analyzer dump, 
                                         subtype=0/1/2/3 */
   IOEDAS_DT3=3,                      /* buffer events, Random scan mode, 
                                         subtype=0 */
   IOEDAS_DT4=4,                      /* subtype=0 VSAM protocol key
                                         subtype=1 VSAM contents key
					 subtype=2 time stamp (like file header)   
                                      */
   IOEDAS_DT7=7,                      /* strangers, subtype=any */
   IOEDAS_REPLY=8,                    /* reply message, 
                                         subtype=0: OK
					 subtype=1: error 
				      */
   IOEDAS_EOR=99,                     /* end-of-run message, subtype=0 */
};				      
                                      /* buffer header */
struct SIOEDASBH {
   unsigned char ucPreXFer[32];       /* pre-transfer header */
   struct SIOEDASDBH sDBH;            /* datablock header */     
};				      				      				      

                                      /* file header */
struct SIOEDASFH {
   struct SIOEDASBH sBH;              /* buffer header */
   short iType;                       /* header type, =0 */
   short iFile;                       /* file number on tape, 1..n */
   unsigned char ucTapeLabel[6];
   unsigned char ucSpare[2];
   unsigned char ucDate[7];
   unsigned char ucCode;
   unsigned long ulTimeOfDay;
   unsigned char ucRun[8]; 
   unsigned char ucExp[8];
   short iComLen;
   unsigned char ucComment[4010];        
};				      				      				      
                                      /* file data */
struct SIOEDASFD {
   struct SIOEDASBH sBH;              /* buffer header */
   unsigned char ucData[4052];        
};				      				      				      
                                      /* function prototypes */
int _ioedasbh( struct SIOEDASBH *, unsigned long );
int _srt00( struct SIOEDASFH *, struct SIOFH *, unsigned long );
int _srt1x( struct SIOEDASBH *, unsigned long );
int _srt3x( struct SIOEDASBH *, unsigned long );
int _srt4x( struct SIOEDASBH *, unsigned long );
int _srt7x( struct SIOEDASBH *, unsigned long );
int _srt8x( struct SIOEDASBH *, unsigned long );
int _srt99( struct SIOEDASBH *, unsigned long );
				      

#endif
